import 'dart:io';
import 'dart:convert';


class Phone{
    List<int> _content;

    String version;
    int firstOffset;
    int totalLength;

    int charLength = 1; // 字符长度
    int intLength = 4; // 字节长度
    int versionFormatLength = 4;  // 版本信息前4个字节
    int headFormatLength = 8;  // 头部为8个字节
    int phoneFormatLength = 9;  // 每个索引的长度

    Phone({String datFile = "phone.dat"}) {
        File file = File(datFile);
        _content = file.readAsBytesSync();
        version = getVersion(_content);
        firstOffset = getFirstOffset(_content);
        totalLength = (_content.length - firstOffset) ~/ phoneFormatLength;
    }

    getVersion(List<int> content){
        return Utf8Decoder().convert(_content.sublist(0, versionFormatLength));
    }

    get4(List<int> _list){
        if (_list.length < 4){
            return 0;
        }else {
            return _list[0] | _list[1] << 8 | _list[2] << 16 | _list[3] << 24;
        }
    }

    getFirstOffset(List<int> content){
        List<int> _list = content.sublist(versionFormatLength, headFormatLength);
        return get4(_list);
    }

    find(String number){
        if(number.length < 7 || number.length > 11){
            throw "illegal phone length";
        }

        // 二分法查找
        int left = 0;
        int right = totalLength;
        int middle;
        int numberSeven = num.parse(number.substring(0, 7));

        int currentOffset;
        int currentPhone;
        int recordOffset;
        var cardType;
        List<int> recordContent;
        int endOffset;
        List<String> result;

        Map cardTypeMap = {
            1: "移动",
            2: "联通",
            3: "电信",
            4: "电信虚拟运营商",
            5: "联通虚拟运营商",
            6: "移动虚拟运营商"
        };

        while(left < right){
            middle = (left + right) ~/ 2;
            currentOffset = firstOffset + middle * phoneFormatLength;
            if(currentOffset >= _content.length){
                return;
            }

            currentPhone = get4(_content.sublist(currentOffset, currentOffset + intLength));
            recordOffset = get4(_content.sublist(currentOffset + intLength, currentOffset + intLength * 2));
            cardType = _content.sublist(currentOffset + intLength * 2, currentOffset + phoneFormatLength)[0];

            if (currentPhone > numberSeven){
                right -= 1;
            }else if (currentPhone < numberSeven){
                left += 1;
            }else{
                recordContent = _content.sublist(recordOffset);
                endOffset = String.fromCharCodes(recordContent).indexOf("\x00");
                result = Utf8Decoder().convert(recordContent.sublist(0, endOffset)).split("|");
                return {"phoneNum": number,
                    "cardType": cardTypeMap[cardType],
                    "province": result[0],
                    "city": result[1],
                    "zipCode": result[2],
                    "areaZone": result[3]
                };
            }
        }

    }
}
